#!/usr/bin/perl -w
##
# DCPU-16 definitions.
#
# Taken from the specification at http://0x10c.com/doc/dcpu-16.txt
#
# We provide package variables for all of the definitions we use in the
# assembler kit.
#
# @file
# @author Justin Fletcher
#

package DCPU;

## Mapping from register name to instruction value
our %RegisterMap = (
        'A' => 0,
        'B' => 1,
        'C' => 2,
        'X' => 3, 
        'Y' => 4, 
        'Z' => 5, 
        'I' => 6, 
        'J' => 7
    );

## Inverse mapping of instruction value to register
our %InvRegisterMap = map { $RegisterMap{$_} => $_ } keys %RegisterMap;

## Basic opcode mapping
our %BasicOps = (
        'EXT'  => 0, # Extended opcode
        'SET'  => 1,
        'ADD'  => 2,
        'SUB'  => 3,
        'MUL'  => 4,
        'DIV'  => 5,
        'MOD'  => 6,
        'SHL'  => 7,
        'SHR'  => 8,
        'AND'  => 9,
        'BOR'  => 10,
        'XOR'  => 11,
        'IFE'  => 12,
        'IFN'  => 13,
        'IFG'  => 14,
        'IFB'  => 15,
    );

## Inverse basic opcode mapping
our %InvBasicOps = map { $BasicOps{$_} => $_ } keys %BasicOps;

## Extended opcode, mapped into opcode 0
our %ExtendedOps = (
        # ExtOpcode 0 is reserved
        'JSR'  => 1,
        # ExtOpcode 2-15 is reserved
    );

## Inverse basic opcode mapping
our %InvExtendedOps = map { $ExtendedOps{$_} => $_ } keys %ExtendedOps;


1;