#!/usr/bin/perl -w
##
# DCPU-16 assembler kit.
#
# based on specification http://0x10c.com/doc/dcpu-16.txt
#
# Syntax: dasm.pl <source> <object>
#
# Example:
#   ./dasm.pl <assembler>.dasm <output>.dump
#       - Assemble and produce a binary dump
#   ./dasm.pl <assembler>.dasm
#       - Assemble and produce an internal structure dump
#   ./dasm.pl <output>.dump <assembler>.dasm
#       - Disassemble into source
#
#
# Source format:
# [<label>]    <opcode>       <operand a>, <operand b>
# [<label>]    <ext-opcode>   <operand>
# ; <comment>
#
# @file
# @author Justin Fletcher
# 

use Getopt::Long;

use DASM;

## Input filename
my $infile;

## Output filename
my $outfile;

# Create us an assembler data structure
my $dasm = new DASM();

GetOptions("input=s" => \$infile,
           "outfile=s" => \$outfile,
           "help" => sub {
                    print <<EOM;
DASM - DCPU-16 assembler/disassembler kit, by Justin Fletcher.
Syntax: ./dasm.pl [<options>] <input> [<output>]

Options:
    -input <file>  Input file
    -output <file> Output file
    -help          This help message

File formats and extensions:
EOM
                    for my $io (@{ $dasm->listio() })
                    {
                        printf "    %-12s %s\n",
                               $io->{'name'},
                               (join ", ", map { ".$_" } @{ $io->extensions() });
                    }
                    exit(1);
                }
          );
           
if (!defined $infile)
{
    $infile = shift || die "Syntax: $0 [options] <infile> [<outfile>]\n";
}
if (!defined $outfile)
{
    $outfile = shift;
}

# Read in the file (whatever it might be)
$dasm->readfile($infile);

if (!defined $outfile)
{
    # They didn't supply an output file, so just dump what we have
    $dasm->dump();
}
else
{
    # Write out the file
    $dasm->writefile($outfile);
}
